@@ -15,8 +15,8 @@
 
 from libcloud.compute.providers import Provider
 from libcloud.common.cloudstack import CloudStackDriverMixIn
-from libcloud.compute.base import Node, NodeDriver, NodeImage, NodeLocation, \
-                                  NodeSize
+from libcloud.compute.base import Node, NodeDriver, NodeImage, NodeLocation,\
+    NodeSize, StorageVolume
 from libcloud.compute.types import NodeState, LibcloudError
 
 
@@ -78,6 +78,19 @@ class CloudStackForwardingRule(object):
         return self.__class__ is other.__class__ and self.id == other.id
 
 
+class CloudStackDiskOffering(object):
+    """A disk offering within CloudStack."""
+
+    def __init__(self, id, name, size, customizable):
+        self.id = id
+        self.name = name
+        self.size = size
+        self.customizable = customizable
+
+    def __eq__(self, other):
+        return self.__class__ is other.__class__ and self.id == other.id
+
+
 class CloudStackNodeDriver(CloudStackDriverMixIn, NodeDriver):
     """Driver for the CloudStack API.
 
@@ -89,6 +102,7 @@ class CloudStackNodeDriver(CloudStackDriverMixIn, NodeDriver):
 
     name = 'CloudStack'
     api_name = 'cloudstack'
+    website = 'http://cloudstack.org/'
     type = Provider.CLOUDSTACK
 
     NODE_STATE_MAP = {
@@ -101,6 +115,9 @@ class CloudStackNodeDriver(CloudStackDriverMixIn, NodeDriver):
 
     def __init__(self, key, secret=None, secure=True, host=None,
                  path=None, port=None, *args, **kwargs):
+        """
+        @requires: key, secret, host or path
+        """
         host = host if host else self.host
 
         if path is not None:
@@ -124,7 +141,7 @@ class CloudStackNodeDriver(CloudStackDriverMixIn, NodeDriver):
             args['zoneid'] = location.id
         imgs = self._sync_request('listTemplates', **args)
         images = []
-        for img in imgs['template']:
+        for img in imgs.get('template', []):
             images.append(NodeImage(img['id'], img['name'], self, {
                 'hypervisor': img['hypervisor'],
                 'format': img['format'],
@@ -168,9 +185,7 @@ class CloudStackNodeDriver(CloudStackDriverMixIn, NodeDriver):
                 public_ips=public_ips.get(vm['id'], {}).keys(),
                 private_ips=private_ips,
                 driver=self,
-                extra={
-                    'zoneid': vm['zoneid'],
-                }
+                extra={'zoneid': vm['zoneid'], }
             )
 
             addrs = public_ips.get(vm['id'], {}).items()
@@ -205,22 +220,13 @@ class CloudStackNodeDriver(CloudStackDriverMixIn, NodeDriver):
         if location is None:
             location = self.list_locations()[0]
 
-        network_id = kwargs.pop('network_id', None)
-        if network_id is None:
-            networks = self._sync_request('listNetworks')
-
-            if networks:
-                extra_args['networkids'] = networks['network'][0]['id']
-        else:
+        if 'network_id' in kwargs:
             extra_args['networkids'] = network_id
 
-        result = self._async_request('deployVirtualMachine',
-            name=name,
-            displayname=name,
-            serviceofferingid=size.id,
-            templateid=image.id,
-            zoneid=location.id,
-            **extra_args
+        result = self._async_request(
+            'deployVirtualMachine', name=name, displayname=name,
+            serviceofferingid=size.id, templateid=image.id,
+            zoneid=location.id, **extra_args
         )
 
         node = result['virtualmachine']
@@ -247,14 +253,82 @@ class CloudStackNodeDriver(CloudStackDriverMixIn, NodeDriver):
         self._async_request('rebootVirtualMachine', id=node.id)
         return True
 
+    def ex_list_disk_offerings(self):
+        """Fetch a list of all available disk offerings.
+
+        @rtype: C{list} of L{CloudStackDiskOffering}
+        """
+
+        diskOfferings = []
+
+        diskOfferResponse = self._sync_request('listDiskOfferings')
+        for diskOfferDict in diskOfferResponse.get('diskoffering', ()):
+            diskOfferings.append(
+                CloudStackDiskOffering(
+                    id=diskOfferDict['id'],
+                    name=diskOfferDict['name'],
+                    size=diskOfferDict['disksize'],
+                    customizable=diskOfferDict['iscustomized']))
+
+        return diskOfferings
+
+    def create_volume(self, size, name, location, snapshot=None):
+        # TODO Add snapshot handling
+        for diskOffering in self.ex_list_disk_offerings():
+            if diskOffering.size == size or diskOffering.customizable:
+                break
+        else:
+            raise LibcloudError(
+                'Disk offering with size=%s not found' % size)
+
+        extraParams = dict()
+        if diskOffering.customizable:
+            extraParams['size'] = size
+
+        requestResult = self._async_request('createVolume',
+                                            name=name,
+                                            diskOfferingId=diskOffering.id,
+                                            zoneId=location.id,
+                                            **extraParams)
+
+        volumeResponse = requestResult['volume']
+
+        return StorageVolume(id=volumeResponse['id'],
+                             name=name,
+                             size=size,
+                             driver=self,
+                             extra=dict(name=volumeResponse['name']))
+
+    def attach_volume(self, node, volume, device=None):
+        # TODO Add handling for device name
+        self._async_request('attachVolume', id=volume.id,
+                            virtualMachineId=node.id)
+        return True
+
+    def detach_volume(self, volume):
+        self._async_request('detachVolume', id=volume.id)
+        return True
+
+    def destroy_volume(self, volume):
+        self._sync_request('deleteVolume', id=volume.id)
+        return True
+
     def ex_allocate_public_ip(self, node):
-        "Allocate a public IP and bind it to a node."
+        """
+        "Allocate a public IP and bind it to a node.
+
+        @param      node: Node which should be used
+        @type       node: L{Node}
+
+        @rtype: L{CloudStackAddress}
+        """
 
         zoneid = node.extra['zoneid']
         addr = self._async_request('associateIpAddress', zoneid=zoneid)
         addr = addr['ipaddress']
-        result = self._sync_request('enableStaticNat', virtualmachineid=node.id,
-                                   ipaddressid=addr['id'])
+        result = self._sync_request('enableStaticNat',
+                                    virtualmachineid=node.id,
+                                    ipaddressid=addr['id'])
         if result.get('success', '').lower() != 'true':
             return None
 
@@ -264,7 +338,17 @@ class CloudStackNodeDriver(CloudStackDriverMixIn, NodeDriver):
         return addr
 
     def ex_release_public_ip(self, node, address):
-        "Release a public IP."
+        """
+        Release a public IP.
+
+        @param      node: Node which should be used
+        @type       node: L{Node}
+
+        @param      address: CloudStackAddress which should be used
+        @type       address: L{CloudStackAddress}
+
+        @rtype: C{bool}
+        """
 
         node.extra['ip_addresses'].remove(address)
         node.public_ips.remove(address.address)
@@ -275,7 +359,26 @@ class CloudStackNodeDriver(CloudStackDriverMixIn, NodeDriver):
 
     def ex_add_ip_forwarding_rule(self, node, address, protocol,
                                   start_port, end_port=None):
-        "Add a NAT/firewall forwarding rule."
+        """
+        "Add a NAT/firewall forwarding rule.
+
+        @param      node: Node which should be used
+        @type       node: L{Node}
+
+        @param      address: CloudStackAddress which should be used
+        @type       address: L{CloudStackAddress}
+
+        @param      protocol: Protocol which should be used (TCP or UDP)
+        @type       protocol: C{str}
+
+        @param      start_port: Start port which should be used
+        @type       start_port: C{int}
+
+        @param end_port: End port which should be used
+        @type end_port: C{int}
+
+        @rtype: L{CloudStackForwardingRule}
+        """
 
         protocol = protocol.upper()
         if protocol not in ('TCP', 'UDP'):
@@ -297,14 +400,37 @@ class CloudStackNodeDriver(CloudStackDriverMixIn, NodeDriver):
         return rule
 
     def ex_delete_ip_forwarding_rule(self, node, rule):
-        "Remove a NAT/firewall forwarding rule."
+        """
+        Remove a NAT/firewall forwarding rule.
+
+        @param      node: Node which should be used
+        @type       node: L{Node}
+
+        @param rule: Forwarding rule which should be used
+        @type rule: L{CloudStackForwardingRule}
+
+        @rtype: C{bool}
+        """
 
         node.extra['ip_forwarding_rules'].remove(rule)
         self._async_request('deleteIpForwardingRule', id=rule.id)
         return True
 
     def ex_register_iso(self, name, url, location=None, **kwargs):
-        "Registers an existing ISO by URL."
+        """
+        Registers an existing ISO by URL.
+
+        @param      name: Name which should be used
+        @type       name: C{str}
+
+        @param      url: Url should be used
+        @type       url: C{str}
+
+        @param      location: Location which should be used
+        @type       location: L{NodeLocation}
+
+        @rtype: C{str}
+        """
         if location is None:
             location = self.list_locations()[0]
 
