@@ -33,6 +33,8 @@ class LibvirtNodeDriver(NodeDriver):
     """
 
     type = Provider.LIBVIRT
+    name = 'Libvirt'
+    website = 'http://libvirt.org/'
 
     NODE_STATE_MAP = {
         0: NodeState.TERMINATED,
@@ -46,6 +48,9 @@ class LibvirtNodeDriver(NodeDriver):
     }
 
     def __init__(self, uri):
+        """
+        @requires: uri
+        """
         if not have_libvirt:
             raise RuntimeError('Libvirt driver requires \'libvirt\' Python ' +
                                'package')
@@ -89,6 +94,11 @@ class LibvirtNodeDriver(NodeDriver):
     def ex_start(self, node):
         """
         Start a stopped node.
+
+        @param  node: Node which should be used
+        @type   node: L{Node}
+
+        @rtype: C{bool}
         """
         domain = self._get_domain_for_node(node=node)
         return domain.create() == 0
@@ -96,6 +106,11 @@ class LibvirtNodeDriver(NodeDriver):
     def ex_shutdown(self, node):
         """
         Shutdown a running node.
+
+        @param  node: Node which should be used
+        @type   node: L{Node}
+
+        @rtype: C{bool}
         """
         domain = self._get_domain_for_node(node=node)
         return domain.shutdown() == 0
@@ -103,6 +118,11 @@ class LibvirtNodeDriver(NodeDriver):
     def ex_suspend(self, node):
         """
         Suspend a running node.
+
+        @param  node: Node which should be used
+        @type   node: L{Node}
+
+        @rtype: C{bool}
         """
         domain = self._get_domain_for_node(node=node)
         return domain.suspend() == 0
@@ -110,6 +130,11 @@ class LibvirtNodeDriver(NodeDriver):
     def ex_resume(self, node):
         """
         Resume a suspended node.
+
+        @param  node: Node which should be used
+        @type   node: L{Node}
+
+        @rtype: C{bool}
         """
         domain = self._get_domain_for_node(node=node)
         return domain.resume() == 0
