@@ -15,9 +15,9 @@
 
 """libcloud driver for the Linode(R) API
 
-This driver implements all libcloud functionality for the Linode API.  Since the
-API is a bit more fine-grained, create_node abstracts a significant amount of
-work (and may take a while to run).
+This driver implements all libcloud functionality for the Linode API.
+Since the API is a bit more fine-grained, create_node abstracts a significant
+amount of work (and may take a while to run).
 
 Linode home page                    http://www.linode.com/
 Linode API documentation            http://www.linode.com/api/
@@ -41,8 +41,8 @@ from copy import copy
 
 from libcloud.utils.py3 import PY3
 
-from libcloud.common.linode import (API_ROOT, LinodeException, LinodeConnection,
-    LINODE_PLAN_IDS)
+from libcloud.common.linode import (API_ROOT, LinodeException,
+                                    LinodeConnection, LINODE_PLAN_IDS)
 from libcloud.compute.types import Provider, NodeState
 from libcloud.compute.base import NodeDriver, NodeSize, Node, NodeLocation
 from libcloud.compute.base import NodeAuthPassword, NodeAuthSSHKey
@@ -71,6 +71,7 @@ class LinodeNodeDriver(NodeDriver):
     """
     type = Provider.LINODE
     name = "Linode"
+    website = 'http://www.linode.com/'
     connectionCls = LinodeConnection
     _linode_plan_ids = LINODE_PLAN_IDS
 
@@ -78,41 +79,52 @@ class LinodeNodeDriver(NodeDriver):
         """Instantiate the driver with the given API key
 
         @keyword key: the API key to use
-        @type key: C{str}"""
+        @type key: C{str}
+
+        @requires: key
+        """
         self.datacenter = None
         NodeDriver.__init__(self, key)
 
     # Converts Linode's state from DB to a NodeState constant.
     LINODE_STATES = {
-        -2: NodeState.UNKNOWN,              # Boot Failed
-        -1: NodeState.PENDING,              # Being Created
-         0: NodeState.PENDING,              # Brand New
-         1: NodeState.RUNNING,              # Running
-         2: NodeState.TERMINATED,           # Powered Off
-         3: NodeState.REBOOTING,            # Shutting Down
-         4: NodeState.UNKNOWN               # Reserved
+        -2: NodeState.UNKNOWN,    # Boot Failed
+        -1: NodeState.PENDING,    # Being Created
+        0: NodeState.PENDING,     # Brand New
+        1: NodeState.RUNNING,     # Running
+        2: NodeState.TERMINATED,  # Powered Off
+        3: NodeState.REBOOTING,   # Shutting Down
+        4: NodeState.UNKNOWN      # Reserved
     }
 
     def list_nodes(self):
-        """List all Linodes that the API key can access
+        """
+        List all Linodes that the API key can access
 
-        This call will return all Linodes that the API key in use has access to.
+        This call will return all Linodes that the API key in use has access
+         to.
         If a node is in this list, rebooting will work; however, creation and
         destruction are a separate grant.
 
-        @return: C{list} of L{Node} objects that the API key can access"""
+        @return: List of node objects that the API key can access
+        @rtype: C{list} of L{Node}
+        """
         params = {"api_action": "linode.list"}
         data = self.connection.request(API_ROOT, params=params).objects[0]
         return self._to_nodes(data)
 
     def reboot_node(self, node):
-        """Reboot the given Linode
+        """
+        Reboot the given Linode
 
         Will issue a shutdown job followed by a boot job, using the last booted
         configuration.  In most cases, this will be the only configuration.
 
-        @keyword node: the Linode to reboot
-        @type node: L{Node}"""
+        @param      node: the Linode to reboot
+        @type       node: L{Node}
+
+        @rtype: C{bool}
+        """
         params = {"api_action": "linode.reboot", "LinodeID": node.id}
         self.connection.request(API_ROOT, params=params)
         return True
@@ -128,10 +140,13 @@ class LinodeNodeDriver(NodeDriver):
         Linode can be removed; however, this call explicitly skips those
         safeguards. There is no going back from this method.
 
-        @keyword node: the Linode to destroy
-        @type node: L{Node}"""
+        @param       node: the Linode to destroy
+        @type        node: L{Node}
+
+        @rtype: C{bool}
+        """
         params = {"api_action": "linode.delete", "LinodeID": node.id,
-            "skipChecks": True}
+                  "skipChecks": True}
         self.connection.request(API_ROOT, params=params)
         return True
 
@@ -146,48 +161,49 @@ class LinodeNodeDriver(NodeDriver):
         prevent a runaway script from ruining your day.
 
         @keyword name: the name to assign the Linode (mandatory)
-        @type name: C{str}
+        @type    name: C{str}
 
         @keyword image: which distribution to deploy on the Linode (mandatory)
-        @type image: L{NodeImage}
+        @type    image: L{NodeImage}
 
         @keyword size: the plan size to create (mandatory)
-        @type size: L{NodeSize}
+        @type    size: L{NodeSize}
 
         @keyword auth: an SSH key or root password (mandatory)
-        @type auth: L{NodeAuthSSHKey} or L{NodeAuthPassword}
+        @type    auth: L{NodeAuthSSHKey} or L{NodeAuthPassword}
 
         @keyword location: which datacenter to create the Linode in
-        @type location: L{NodeLocation}
+        @type    location: L{NodeLocation}
 
         @keyword ex_swap: size of the swap partition in MB (128)
-        @type ex_swap: C{int}
+        @type    ex_swap: C{int}
 
         @keyword ex_rsize: size of the root partition in MB (plan size - swap).
-        @type ex_rsize: C{int}
+        @type    ex_rsize: C{int}
 
         @keyword ex_kernel: a kernel ID from avail.kernels (Latest 2.6 Stable).
-        @type ex_kernel: C{str}
+        @type    ex_kernel: C{str}
 
         @keyword ex_payment: one of 1, 12, or 24; subscription length (1)
-        @type ex_payment: C{int}
+        @type    ex_payment: C{int}
 
         @keyword ex_comment: a small comment for the configuration (libcloud)
-        @type ex_comment: C{str}
+        @type    ex_comment: C{str}
 
         @keyword ex_private: whether or not to request a private IP (False)
-        @type ex_private: C{bool}
+        @type    ex_private: C{bool}
 
         @keyword lconfig: what to call the configuration (generated)
-        @type lconfig: C{str}
+        @type    lconfig: C{str}
 
         @keyword lroot: what to call the root image (generated)
-        @type lroot: C{str}
+        @type    lroot: C{str}
 
         @keyword lswap: what to call the swap space (generated)
-        @type lswap: C{str}
+        @type    lswap: C{str}
 
-        @return: a L{Node} representing the newly-created Linode
+        @return: Node representing the newly-created Linode
+        @rtype: L{Node}
         """
         name = kwargs["name"]
         image = kwargs["image"]
@@ -212,7 +228,8 @@ class LinodeNodeDriver(NodeDriver):
             raise LinodeException(0xFB, "Invalid plan ID -- avail.plans")
 
         # Payment schedule
-        payment = "1" if "ex_payment" not in kwargs else str(kwargs["ex_payment"])
+        payment = "1" if "ex_payment" not in kwargs else \
+            str(kwargs["ex_payment"])
         if payment not in ["1", "12", "24"]:
             raise LinodeException(0xFB, "Invalid subscription (1, 12, 24)")
 
@@ -236,7 +253,7 @@ class LinodeNodeDriver(NodeDriver):
             raise LinodeException(0xFB, "Need an integer swap size")
 
         # Root partition size
-        imagesize = (size.disk - swap) if "ex_rsize" not in kwargs else \
+        imagesize = (size.disk - swap) if "ex_rsize" not in kwargs else\
             int(kwargs["ex_rsize"])
         if (imagesize + swap) > size.disk:
             raise LinodeException(0xFB, "Total disk images are too big")
@@ -255,13 +272,13 @@ class LinodeNodeDriver(NodeDriver):
                 kernel = 111 if image.extra['pvops'] else 107
             else:
                 kernel = 110 if image.extra['pvops'] else 60
-        params = { "api_action": "avail.kernels" }
+        params = {"api_action": "avail.kernels"}
         kernels = self.connection.request(API_ROOT, params=params).objects[0]
         if kernel not in [z["KERNELID"] for z in kernels]:
             raise LinodeException(0xFB, "Invalid kernel -- avail.kernels")
 
         # Comments
-        comments = "Created by Apache libcloud <http://www.libcloud.org>" if \
+        comments = "Created by Apache libcloud <http://www.libcloud.org>" if\
             "ex_comment" not in kwargs else kwargs["ex_comment"]
 
         # Labels
@@ -276,13 +293,13 @@ class LinodeNodeDriver(NodeDriver):
 
         # Step 1: linode.create
         params = {
-            "api_action":   "linode.create",
+            "api_action": "linode.create",
             "DatacenterID": chosen,
-            "PlanID":       size.id,
-            "PaymentTerm":  payment
+            "PlanID": size.id,
+            "PaymentTerm": payment
         }
         data = self.connection.request(API_ROOT, params=params).objects[0]
-        linode = { "id": data["LinodeID"] }
+        linode = {"id": data["LinodeID"]}
 
         # Step 1b. linode.update to rename the Linode
         params = {
@@ -295,8 +312,8 @@ class LinodeNodeDriver(NodeDriver):
         # Step 1c. linode.ip.addprivate if it was requested
         if "ex_private" in kwargs and kwargs["ex_private"]:
             params = {
-                "api_action":   "linode.ip.addprivate",
-                "LinodeID":     linode["id"]
+                "api_action": "linode.ip.addprivate",
+                "LinodeID": linode["id"]
             }
             self.connection.request(API_ROOT, params=params)
 
@@ -305,12 +322,12 @@ class LinodeNodeDriver(NodeDriver):
             root = binascii.b2a_base64(os.urandom(8)).decode('ascii')
 
         params = {
-            "api_action":       "linode.disk.createfromdistribution",
-            "LinodeID":         linode["id"],
-            "DistributionID":   image.id,
-            "Label":            label["lroot"],
-            "Size":             imagesize,
-            "rootPass":         root,
+            "api_action": "linode.disk.createfromdistribution",
+            "LinodeID": linode["id"],
+            "DistributionID": image.id,
+            "Label": label["lroot"],
+            "Size": imagesize,
+            "rootPass": root,
         }
         if ssh:
             params["rootSSHKey"] = ssh
@@ -319,11 +336,11 @@ class LinodeNodeDriver(NodeDriver):
 
         # Step 3: linode.disk.create for swap
         params = {
-            "api_action":       "linode.disk.create",
-            "LinodeID":         linode["id"],
-            "Label":            label["lswap"],
-            "Type":             "swap",
-            "Size":             swap
+            "api_action": "linode.disk.create",
+            "LinodeID": linode["id"],
+            "Label": label["lswap"],
+            "Type": "swap",
+            "Size": swap
         }
         data = self.connection.request(API_ROOT, params=params).objects[0]
         linode["swapimage"] = data["DiskID"]
@@ -331,26 +348,26 @@ class LinodeNodeDriver(NodeDriver):
         # Step 4: linode.config.create for main profile
         disks = "%s,%s,,,,,,," % (linode["rootimage"], linode["swapimage"])
         params = {
-            "api_action":       "linode.config.create",
-            "LinodeID":         linode["id"],
-            "KernelID":         kernel,
-            "Label":            label["lconfig"],
-            "Comments":         comments,
-            "DiskList":         disks
+            "api_action": "linode.config.create",
+            "LinodeID": linode["id"],
+            "KernelID": kernel,
+            "Label": label["lconfig"],
+            "Comments": comments,
+            "DiskList": disks
         }
         data = self.connection.request(API_ROOT, params=params).objects[0]
         linode["config"] = data["ConfigID"]
 
         # Step 5: linode.boot
         params = {
-            "api_action":       "linode.boot",
-            "LinodeID":         linode["id"],
-            "ConfigID":         linode["config"]
+            "api_action": "linode.boot",
+            "LinodeID": linode["id"],
+            "ConfigID": linode["config"]
         }
         self.connection.request(API_ROOT, params=params)
 
         # Make a node out of it and hand it back
-        params = { "api_action": "linode.list", "LinodeID": linode["id"] }
+        params = {"api_action": "linode.list", "LinodeID": linode["id"]}
         data = self.connection.request(API_ROOT, params=params).objects[0]
         nodes = self._to_nodes(data)
 
@@ -360,33 +377,37 @@ class LinodeNodeDriver(NodeDriver):
         return None
 
     def list_sizes(self, location=None):
-        """List available Linode plans
+        """
+        List available Linode plans
 
         Gets the sizes that can be used for creating a Linode.  Since available
         Linode plans vary per-location, this method can also be passed a
         location to filter the availability.
 
         @keyword location: the facility to retrieve plans in
-        @type location: NodeLocation
+        @type    location: L{NodeLocation}
 
-        @return: a C{list} of L{NodeSize}s"""
-        params = { "api_action": "avail.linodeplans" }
+        @rtype: C{list} of L{NodeSize}
+        """
+        params = {"api_action": "avail.linodeplans"}
         data = self.connection.request(API_ROOT, params=params).objects[0]
         sizes = []
         for obj in data:
             n = NodeSize(id=obj["PLANID"], name=obj["LABEL"], ram=obj["RAM"],
-                    disk=(obj["DISK"] * 1024), bandwidth=obj["XFER"],
-                    price=obj["PRICE"], driver=self.connection.driver)
+                         disk=(obj["DISK"] * 1024), bandwidth=obj["XFER"],
+                         price=obj["PRICE"], driver=self.connection.driver)
             sizes.append(n)
         return sizes
 
     def list_images(self):
-        """List available Linux distributions
+        """
+        List available Linux distributions
 
         Retrieve all Linux distributions that can be deployed to a Linode.
 
-        @return: a C{list} of L{NodeImage}s"""
-        params = { "api_action": "avail.distributions" }
+        @rtype: C{list} of L{NodeImage}
+        """
+        params = {"api_action": "avail.distributions"}
         data = self.connection.request(API_ROOT, params=params).objects[0]
         distros = []
         for obj in data:
@@ -399,12 +420,14 @@ class LinodeNodeDriver(NodeDriver):
         return distros
 
     def list_locations(self):
-        """List available facilities for deployment
+        """
+        List available facilities for deployment
 
         Retrieve all facilities that a Linode can be deployed in.
 
-        @return: a C{list} of L{NodeLocation}s"""
-        params = { "api_action": "avail.datacenters" }
+        @rtype: C{list} of L{NodeLocation}
+        """
+        params = {"api_action": "avail.datacenters"}
         data = self.connection.request(API_ROOT, params=params).objects[0]
         nl = []
         for dc in data:
@@ -424,16 +447,20 @@ class LinodeNodeDriver(NodeDriver):
         return nl
 
     def linode_set_datacenter(self, dc):
-        """Set the default datacenter for Linode creation
+        """
+        Set the default datacenter for Linode creation
 
         Since Linodes must be created in a facility, this function sets the
         default that L{create_node} will use.  If a C{location} keyword is not
         passed to L{create_node}, this method must have already been used.
 
         @keyword dc: the datacenter to create Linodes in unless specified
-        @type dc: L{NodeLocation}"""
+        @type dc: L{NodeLocation}
+
+        @rtype: C{bool}
+        """
         did = dc.id
-        params = { "api_action": "avail.datacenters" }
+        params = {"api_action": "avail.datacenters"}
         data = self.connection.request(API_ROOT, params=params).objects[0]
         for datacenter in data:
             if did == dc["DATACENTERID"]:
@@ -457,8 +484,9 @@ class LinodeNodeDriver(NodeDriver):
         for o in objs:
             lid = o["LINODEID"]
             nodes[lid] = n = Node(id=lid, name=o["LABEL"], public_ips=[],
-                private_ips=[], state=self.LINODE_STATES[o["STATUS"]],
-                driver=self.connection.driver)
+                                  private_ips=[],
+                                  state=self.LINODE_STATES[o["STATUS"]],
+                                  driver=self.connection.driver)
             n.extra = copy(o)
             n.extra["PLANID"] = self._linode_plan_ids.get(o.get("TOTALRAM"))
             batch.append({"api_action": "linode.ip.list", "LinodeID": lid})
@@ -474,8 +502,8 @@ class LinodeNodeDriver(NodeDriver):
 
         for twenty_five in izip_longest(*args):
             twenty_five = [q for q in twenty_five if q]
-            params = { "api_action": "batch",
-                "api_requestArray": json.dumps(twenty_five) }
+            params = {"api_action": "batch",
+                      "api_requestArray": json.dumps(twenty_five)}
             req = self.connection.request(API_ROOT, params=params)
             if not req.success() or len(req.objects) == 0:
                 return None
@@ -485,7 +513,7 @@ class LinodeNodeDriver(NodeDriver):
         for ip_list in ip_answers:
             for ip in ip_list:
                 lid = ip["LINODEID"]
-                which = nodes[lid].public_ips if ip["ISPUBLIC"] == 1 else \
+                which = nodes[lid].public_ips if ip["ISPUBLIC"] == 1 else\
                     nodes[lid].private_ips
                 which.append(ip["IPADDRESS"])
         return list(nodes.values())
@@ -501,8 +529,9 @@ def _izip_longest(*args, **kwds):
 
     fillvalue = kwds.get('fillvalue')
 
-    def sentinel(counter = ([fillvalue]*(len(args)-1)).pop):
+    def sentinel(counter=([fillvalue] * (len(args) - 1)).pop):
         yield counter()  # yields the fillvalue, or raises IndexError
+
     fillers = itertools.repeat(fillvalue)
     iters = [itertools.chain(it, sentinel(), fillers) for it in args]
     try:
