@@ -31,8 +31,12 @@ class DummyDNSDriver(DNSDriver):
     """
 
     name = 'Dummy DNS Provider'
+    website = 'http://example.com'
 
     def __init__(self, api_key, api_secret):
+        """
+        @requires: api_key, api_secret
+        """
         self._zones = {}
 
     def list_record_types(self):
