@@ -103,7 +103,7 @@ class ZerigoDNSConnection(ConnectionUserAndKey):
 
     def add_default_headers(self, headers):
         auth_b64 = base64.b64encode(b('%s:%s' % (self.user_id, self.key)))
-        headers['Authorization'] = 'Basic %s' % (auth_b64)
+        headers['Authorization'] = 'Basic %s' % (auth_b64.decode('utf-8'))
         return headers
 
     def request(self, action, params=None, data='', headers=None,
@@ -125,6 +125,7 @@ class ZerigoDNSConnection(ConnectionUserAndKey):
 class ZerigoDNSDriver(DNSDriver):
     type = Provider.ZERIGO
     name = 'Zerigo DNS'
+    website = 'http://www.zerigo.com/'
     connectionCls = ZerigoDNSConnection
 
     RECORD_TYPE_MAP = {
@@ -139,6 +140,8 @@ class ZerigoDNSDriver(DNSDriver):
         RecordType.NS: 'NS',
         RecordType.PTR: 'PTR',
         RecordType.SPF: 'SPF',
+        RecordType.GEO: 'GEO',
+        RecordType.URL: 'URL',
     }
 
     def list_zones(self):
