@@ -76,8 +76,8 @@ class Object(object):
     def get_cdn_url(self):
         return self.driver.get_object_cdn_url(obj=self)
 
-    def enable_cdn(self):
-        return self.driver.enable_object_cdn(obj=self)
+    def enable_cdn(self, **kwargs):
+        return self.driver.enable_object_cdn(obj=self, **kwargs)
 
     def download(self, destination_path, overwrite_existing=False,
                  delete_on_failure=True):
@@ -122,8 +122,8 @@ class Container(object):
     def get_cdn_url(self):
         return self.driver.get_container_cdn_url(container=self)
 
-    def enable_cdn(self):
-        return self.driver.enable_container_cdn(container=self)
+    def enable_cdn(self, **kwargs):
+        return self.driver.enable_container_cdn(container=self, **kwargs)
 
     def get_object(self, object_name):
         return self.driver.get_object(container_name=self.name,
@@ -139,7 +139,9 @@ class Container(object):
 
     def download_object(self, obj, destination_path, overwrite_existing=False,
                         delete_on_failure=True):
-        return self.driver.download_object(obj, destination_path)
+        return self.driver.download_object(
+            obj, destination_path, overwrite_existing=overwrite_existing,
+            delete_on_failure=delete_on_failure)
 
     def download_object_as_stream(self, obj, chunk_size=None):
         return self.driver.download_object_as_stream(obj, chunk_size)
@@ -545,7 +547,7 @@ class StorageDriver(BaseDriver):
             file_size = os.path.getsize(file_path)
             upload_func_kwargs['chunked'] = False
 
-        if file_size:
+        if file_size is not None:
             headers['Content-Length'] = file_size
 
         headers['Content-Type'] = content_type
